@@ -1,8 +1,6 @@
 /*
   configuration.cpp - wraps gpgme configuration components
   Copyright (C) 2010 Klarälvdalens Datakonsult AB
-  2016 Bundesamt für Sicherheit in der Informationstechnik
-  Software engineering by Intevation GmbH
 
   This file is part of GPGME++.
 
@@ -22,9 +20,7 @@
   Boston, MA 02110-1301, USA.
 */
 
-#ifdef HAVE_CONFIG_H
- #include "config.h"
-#endif
+#include <config-gpgme++.h>
 
 #include "configuration.h"
 #include "error.h"
@@ -32,23 +28,24 @@
 
 #include <gpgme.h>
 
+#include <boost/foreach.hpp>
+
 #include <iterator>
 #include <algorithm>
 #include <ostream>
 #include <cstring>
-#include <assert.h>
 
 using namespace GpgME;
 using namespace GpgME::Configuration;
 
-typedef std::shared_ptr< std::remove_pointer<gpgme_conf_opt_t>::type > shared_gpgme_conf_opt_t;
-typedef std::weak_ptr< std::remove_pointer<gpgme_conf_opt_t>::type > weak_gpgme_conf_opt_t;
+typedef boost::shared_ptr< boost::remove_pointer<gpgme_conf_opt_t>::type > shared_gpgme_conf_opt_t;
+typedef boost::weak_ptr< boost::remove_pointer<gpgme_conf_opt_t>::type > weak_gpgme_conf_opt_t;
 
-typedef std::shared_ptr< std::remove_pointer<gpgme_conf_arg_t>::type > shared_gpgme_conf_arg_t;
-typedef std::weak_ptr< std::remove_pointer<gpgme_conf_arg_t>::type > weak_gpgme_conf_arg_t;
+typedef boost::shared_ptr< boost::remove_pointer<gpgme_conf_arg_t>::type > shared_gpgme_conf_arg_t;
+typedef boost::weak_ptr< boost::remove_pointer<gpgme_conf_arg_t>::type > weak_gpgme_conf_arg_t;
 
-typedef std::shared_ptr< std::remove_pointer<gpgme_ctx_t>::type > shared_gpgme_ctx_t;
-typedef std::weak_ptr< std::remove_pointer<gpgme_ctx_t>::type > weak_gpgme_ctx_t;
+typedef boost::shared_ptr< boost::remove_pointer<gpgme_ctx_t>::type > shared_gpgme_ctx_t;
+typedef boost::weak_ptr< boost::remove_pointer<gpgme_ctx_t>::type > weak_gpgme_ctx_t;
 
 namespace
 {
@@ -61,10 +58,11 @@ struct nodelete {
 std::vector<Component> Component::load(Error &returnedError)
 {
 
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     //
     // 1. get a context:
     //
-    gpgme_ctx_t ctx_native = nullptr;
+    gpgme_ctx_t ctx_native = 0;
     if (const gpgme_error_t err = gpgme_new(&ctx_native)) {
         returnedError = Error(err);
         return std::vector<Component>();
@@ -74,7 +72,7 @@ std::vector<Component> Component::load(Error &returnedError)
     //
     // 2. load the config:
     //
-    gpgme_conf_comp_t conf_list_native = nullptr;
+    gpgme_conf_comp_t conf_list_native = 0;
     if (const gpgme_error_t err = gpgme_op_conf_load(ctx_native, &conf_list_native)) {
         returnedError = Error(err);
         return std::vector<Component>();
@@ -94,7 +92,7 @@ std::vector<Component> Component::load(Error &returnedError)
         }
 
         // now prevent double-free of next.get() and following:
-        head->next = nullptr;
+        head->next = 0;
 
         // now add a new Component to 'result' (may throw):
         result.resize(result.size() + 1);
@@ -103,11 +101,16 @@ std::vector<Component> Component::load(Error &returnedError)
     }
 
     return result;
+#else
+    returnedError = Error(make_error(GPG_ERR_NOT_SUPPORTED));
+    return std::vector<Component>();
+#endif
 }
 
 Error Component::save() const
 {
 
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull()) {
         return Error(make_error(GPG_ERR_INV_ARG));
     }
@@ -115,7 +118,7 @@ Error Component::save() const
     //
     // 1. get a context:
     //
-    gpgme_ctx_t ctx_native = nullptr;
+    gpgme_ctx_t ctx_native = 0;
     if (const gpgme_error_t err = gpgme_new(&ctx_native)) {
         return Error(err);
     }
@@ -125,26 +128,42 @@ Error Component::save() const
     // 2. save the config:
     //
     return Error(gpgme_op_conf_save(ctx.get(), comp.get()));
+#else
+    return Error(make_error(GPG_ERR_NOT_SUPPORTED));
+#endif
 }
 
 const char *Component::name() const
 {
-    return comp ? comp->name : nullptr;
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
+    return comp ? comp->name : 0 ;
+#else
+    return 0;
+#endif
 }
 
 const char *Component::description() const
 {
-    return comp ? comp->description : nullptr ;
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
+    return comp ? comp->description : 0 ;
+#else
+    return 0;
+#endif
 }
 
 const char *Component::programName() const
 {
-    return comp ? comp->program_name : nullptr ;
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
+    return comp ? comp->program_name : 0 ;
+#else
+    return 0;
+#endif
 }
 
 Option Component::option(unsigned int idx) const
 {
-    gpgme_conf_opt_t opt = nullptr;
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
+    gpgme_conf_opt_t opt = 0;
     if (comp) {
         opt = comp->options;
     }
@@ -154,13 +173,18 @@ Option Component::option(unsigned int idx) const
     }
     if (opt) {
         return Option(comp, opt);
+    } else {
+#endif
+        return Option();
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     }
-    return Option();
+#endif
 }
 
 Option Component::option(const char *name) const
 {
-    gpgme_conf_opt_t opt = nullptr;
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
+    gpgme_conf_opt_t opt = 0;
     if (comp) {
         opt = comp->options;
     }
@@ -170,41 +194,50 @@ Option Component::option(const char *name) const
     }
     if (opt) {
         return Option(comp, opt);
+    } else {
+#endif
+        return Option();
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     }
-    return Option();
+#endif
 }
 
 unsigned int Component::numOptions() const
 {
     unsigned int result = 0;
-    for (gpgme_conf_opt_t opt = comp ? comp->options : nullptr ; opt ; opt = opt->next) {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
+    for (gpgme_conf_opt_t opt = comp ? comp->options : 0 ; opt ; opt = opt->next) {
         ++result;
     }
+#endif
     return result;
 }
 
 std::vector<Option> Component::options() const
 {
     std::vector<Option> result;
-    for (gpgme_conf_opt_t opt = comp ? comp->options : nullptr ; opt ; opt = opt->next) {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
+    for (gpgme_conf_opt_t opt = comp ? comp->options : 0 ; opt ; opt = opt->next) {
         result.push_back(Option(comp, opt));
     }
+#endif
     return result;
 }
 
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
 static gpgme_conf_arg_t mygpgme_conf_arg_copy(gpgme_conf_arg_t other, gpgme_conf_type_t type)
 {
-    gpgme_conf_arg_t result = nullptr, last = nullptr;
+    gpgme_conf_arg_t result = 0, last = 0;
     for (gpgme_conf_arg_t a = other ; a ; a = a->next) {
-        gpgme_conf_arg_t arg = nullptr;
+        gpgme_conf_arg_t arg = 0;
         const gpgme_error_t err
             = gpgme_conf_arg_new(&arg, type,
-                                 a->no_arg                 ? nullptr :
+                                 a->no_arg                 ? 0 :
                                  type == GPGME_CONF_STRING ? a->value.string :
                                  /* else */                  static_cast<void *>(&a->value));
         if (err) {
             gpgme_conf_arg_release(result, type);
-            return nullptr;
+            return 0;
         }
         assert(arg);
         if (result) {
@@ -216,6 +249,7 @@ static gpgme_conf_arg_t mygpgme_conf_arg_copy(gpgme_conf_arg_t other, gpgme_conf
     }
     return result;
 }
+#endif
 
 Component Option::parent() const
 {
@@ -224,78 +258,279 @@ Component Option::parent() const
 
 unsigned int Option::flags() const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     return isNull() ? 0 : opt->flags;
+#else
+    return 0;
+#endif
 }
 
 Level Option::level() const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     return isNull() ? Internal : static_cast<Level>(opt->level) ;
+#else
+    return Internal;
+#endif
 }
 
 const char *Option::name() const
 {
-    return isNull() ? nullptr : opt->name ;
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
+    return isNull() ? 0 : opt->name ;
+#else
+    return 0;
+#endif
 }
 
 const char *Option::description() const
 {
-    return isNull() ? nullptr : opt->description ;
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
+    return isNull() ? 0 : opt->description ;
+#else
+    return 0;
+#endif
 }
 
 const char *Option::argumentName() const
 {
-    return isNull() ? nullptr : opt->argname ;
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
+    return isNull() ? 0 : opt->argname ;
+#else
+    return 0;
+#endif
 }
 
 Type Option::type() const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     return isNull() ? NoType : static_cast<Type>(opt->type) ;
+#else
+    return NoType;
+#endif
 }
 
 Type Option::alternateType() const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     return isNull() ? NoType : static_cast<Type>(opt->alt_type) ;
+#else
+    return NoType;
+#endif
 }
 
+#if 0
+static Option::Variant argument_to_variant(gpgme_conf_type_t type, bool list, gpgme_conf_arg_t arg)
+{
+    assert(arg);
+    switch (type) {
+    case GPGME_CONF_NONE:
+        if (list) {
+            // return the count (number of times set):
+            return arg->value.count;
+        } else {
+            return none;
+        }
+    case GPGME_CONF_INT32:
+        if (list) {
+            std::vector<int> result;
+            for (gpgme_conf_arg_t a = arg ; a ; a = a->next) {
+                result.push_back(a->value.int32);
+            }
+            return result;
+        } else {
+            return arg->value.int32;
+        }
+    case GPGME_CONF_UINT32:
+        if (list) {
+            std::vector<unsigned int> result;
+            for (gpgme_conf_arg_t a = arg ; a ; a = a->next) {
+                result.push_back(a->value.uint32);
+            }
+            return result;
+        } else {
+            return arg->value.uint32;
+        }
+    case GPGME_CONF_FILENAME:
+    case GPGME_CONF_LDAP_SERVER:
+    case GPGME_CONF_KEY_FPR:
+    case GPGME_CONF_PUB_KEY:
+    case GPGME_CONF_SEC_KEY:
+    case GPGME_CONF_ALIAS_LIST:
+    // these should not happen in alt_type, but fall through
+    case GPGME_CONF_STRING:
+        if (list) {
+            std::vector<const char *> result;
+            for (gpgme_conf_arg_t a = arg ; a ; a = a->next) {
+                result.push_back(a->value.string);
+            }
+            return result;
+        } else {
+            return arg->value.string;
+        }
+    }
+    assert(!"Option: unknown alt_type!");
+    return Option::Variant();
+}
+
+namespace
+{
+inline const void *to_void_star(const char *s)
+{
+    return s;
+}
+inline const void *to_void_star(const std::string &s)
+{
+    return s.c_str();
+}
+inline const void *to_void_star(const int &i)
+{
+    return &i;    // const-&: sic!
+}
+inline const void *to_void_star(const unsigned int &i)
+{
+    return &i;    // const-&: sic!
+}
+
+struct VariantToArgumentVisitor : boost::static_visitor<gpgme_conf_arg_t> {
+    static gpgme_conf_arg_t make_argument(gpgme_conf_type_t type, const void *value)
+    {
+        gpgme_conf_arg_t arg = 0;
+#ifdef HAVE_GPGME_CONF_ARG_NEW_WITH_CONST_VALUE
+        if (const gpgme_error_t err = gpgme_conf_arg_new(&arg, type, value)) {
+            return 0;
+        }
+#else
+        if (const gpgme_error_t err = gpgme_conf_arg_new(&arg, type, const_cast<void *>(value))) {
+            return 0;
+        }
+#endif
+        else {
+            return arg;
+        }
+    }
+
+    gpgme_conf_arg_t operator()(bool v) const
+    {
+        return v ? make_argument(0) : 0 ;
+    }
+
+    gpgme_conf_arg_t operator()(const char *s) const
+    {
+        return make_argument(s ? s : "");
+    }
+
+    gpgme_conf_arg_t operator()(const std::string &s) const
+    {
+        return operator()(s.c_str());
+    }
+
+    gpgme_conf_arg_t operator()(int i) const
+    {
+        return make_argument(&i);
+    }
+
+    gpgme_conf_arg_t operator()(unsigned int i) const
+    {
+        return make_argument(&i);
+    }
+
+    template <typename T>
+    gpgme_conf_arg_t operator()(const std::vector<T> &value) const
+    {
+        gpgme_conf_arg_t result = 0;
+        gpgme_conf_arg_t last = 0;
+        for (typename std::vector<T>::const_iterator it = value.begin(), end = value.end() ; it != end ; ++it) {
+            if (gpgme_conf_arg_t arg = make_argument(to_void_star(*it))) {
+                if (last) {
+                    last = last->next = arg;
+                } else {
+                    result = last = arg;
+                }
+            }
+        }
+        return result;
+    }
+
+};
+}
+
+static gpgme_conf_arg_t variant_to_argument(const Option::Variant &value)
+{
+    VariantToArgumentVisitor v;
+    return apply_visitor(v, value);
+}
+
+optional<Option::Variant> Option::defaultValue() const
+{
+    if (isNull()) {
+        return optional<Variant>();
+    } else {
+        return argument_to_variant(opt->alt_type, opt->flags & GPGME_CONF_LIST, opt->default_value);
+    }
+}
+#endif
+
 Argument Option::defaultValue() const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull()) {
         return Argument();
     } else {
         return Argument(comp.lock(), opt, opt->default_value, false);
     }
+#else
+    return Argument();
+#endif
 }
 
 const char *Option::defaultDescription() const
 {
-    return isNull() ? nullptr : opt->default_description ;
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
+    return isNull() ? 0 : opt->default_description ;
+#else
+    return 0;
+#endif
 }
 
 Argument Option::noArgumentValue() const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull()) {
         return Argument();
     } else {
         return Argument(comp.lock(), opt, opt->no_arg_value, false);
     }
+#else
+    return Argument();
+#endif
 }
 
 const char *Option::noArgumentDescription() const
 {
-    return isNull() ? nullptr : opt->no_arg_description ;
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
+    return isNull() ? 0 : opt->no_arg_description ;
+#else
+    return 0;
+#endif
 }
 
 Argument Option::activeValue() const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull()) {
         return Argument();
     } else {
         return Argument(comp.lock(), opt, opt->value, false);
     }
+#else
+    return Argument();
+#endif
 }
 
 Argument Option::currentValue() const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull()) {
         return Argument();
     }
@@ -304,19 +539,27 @@ Argument Option::currentValue() const
         opt->value        ? opt->value :
         /* else */          opt->default_value ;
     return Argument(comp.lock(), opt, arg, false);
+#else
+    return Argument();
+#endif
 }
 
 Argument Option::newValue() const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull()) {
         return Argument();
     } else {
         return Argument(comp.lock(), opt, opt->new_value, false);
     }
+#else
+    return Argument();
+#endif
 }
 
 bool Option::set() const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull()) {
         return false;
     } else if (opt->change_value) {
@@ -324,15 +567,23 @@ bool Option::set() const
     } else {
         return opt->value;
     }
+#else
+    return false;
+#endif
 }
 
 bool Option::dirty() const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     return !isNull() && opt->change_value ;
+#else
+    return false;
+#endif
 }
 
 Error Option::setNewValue(const Argument &argument)
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull()) {
         return Error(make_error(GPG_ERR_INV_ARG));
     } else if (argument.isNull()) {
@@ -342,84 +593,126 @@ Error Option::setNewValue(const Argument &argument)
     } else {
         return Error(make_error(GPG_ERR_ENOMEM));
     }
+#else
+    return Error(make_error(GPG_ERR_NOT_SUPPORTED));
+#endif
 }
 
 Error Option::resetToActiveValue()
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull()) {
         return Error(make_error(GPG_ERR_INV_ARG));
     } else {
-        return Error(gpgme_conf_opt_change(opt, 1, nullptr));
+        return Error(gpgme_conf_opt_change(opt, 1, 0));
     }
+#else
+    return Error(make_error(GPG_ERR_NOT_SUPPORTED));
+#endif
 }
 
 Error Option::resetToDefaultValue()
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull()) {
         return Error(make_error(GPG_ERR_INV_ARG));
     } else {
-        return Error(gpgme_conf_opt_change(opt, 0, nullptr));
+        return Error(gpgme_conf_opt_change(opt, 0, 0));
     }
+#else
+    return Error(make_error(GPG_ERR_NOT_SUPPORTED));
+#endif
 }
 
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
 static gpgme_conf_arg_t make_argument(gpgme_conf_type_t type, const void *value)
 {
-    gpgme_conf_arg_t arg = nullptr;
+    gpgme_conf_arg_t arg = 0;
+#ifdef HAVE_GPGME_CONF_ARG_NEW_WITH_CONST_VALUE
     if (const gpgme_error_t err = gpgme_conf_arg_new(&arg, type, value)) {
-        return nullptr;
-    } else {
+        return 0;
+    }
+#else
+    if (const gpgme_error_t err = gpgme_conf_arg_new(&arg, type, const_cast<void *>(value))) {
+        return 0;
+    }
+#endif
+    else {
         return arg;
     }
 }
+#endif
 
 Argument Option::createNoneArgument(bool set) const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull() || alternateType() != NoType) {
         return Argument();
     } else {
         if (set) {
             return createNoneListArgument(1);
+        } else {
+#endif
+            return Argument();
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
         }
     }
-    return Argument();
+#endif
 }
 
 Argument Option::createStringArgument(const char *value) const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull() || alternateType() != StringType) {
         return Argument();
     } else {
         return Argument(comp.lock(), opt, make_argument(GPGME_CONF_STRING, value), true);
     }
+#else
+    return Argument();
+#endif
 }
 
 Argument Option::createStringArgument(const std::string &value) const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull() || alternateType() != StringType) {
         return Argument();
     } else {
         return Argument(comp.lock(), opt, make_argument(GPGME_CONF_STRING, value.c_str()), true);
     }
+#else
+    return Argument();
+#endif
 }
 
 Argument Option::createIntArgument(int value) const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull() || alternateType() != IntegerType) {
         return Argument();
     } else {
         return Argument(comp.lock(), opt, make_argument(GPGME_CONF_INT32, &value), true);
     }
+#else
+    return Argument();
+#endif
 }
 
 Argument Option::createUIntArgument(unsigned int value) const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull() || alternateType() != UnsignedIntegerType) {
         return Argument();
     } else {
         return Argument(comp.lock(), opt, make_argument(GPGME_CONF_UINT32, &value), true);
     }
+#else
+    return Argument();
+#endif
 }
 
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
 namespace
 {
 const void *to_void_star(const char *s)
@@ -442,8 +735,8 @@ const void *to_void_star(const unsigned int &i)
 template <typename T>
 gpgme_conf_arg_t make_argument(gpgme_conf_type_t type, const std::vector<T> &value)
 {
-    gpgme_conf_arg_t result = nullptr;
-    gpgme_conf_arg_t last = nullptr;
+    gpgme_conf_arg_t result = 0;
+    gpgme_conf_arg_t last = 0;
     for (typename std::vector<T>::const_iterator it = value.begin(), end = value.end() ; it != end ; ++it) {
         if (gpgme_conf_arg_t arg = make_argument(type, to_void_star(*it))) {
             if (last) {
@@ -456,39 +749,65 @@ gpgme_conf_arg_t make_argument(gpgme_conf_type_t type, const std::vector<T> &val
     return result;
 }
 }
+#endif
 
 Argument Option::createNoneListArgument(unsigned int value) const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (value) {
         return Argument(comp.lock(), opt, make_argument(GPGME_CONF_NONE, &value), true);
+    } else {
+#endif
+        return Argument();
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     }
-    return Argument();
+#endif
 }
 
 Argument Option::createStringListArgument(const std::vector<const char *> &value) const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     return Argument(comp.lock(), opt, make_argument(GPGME_CONF_STRING, value), true);
+#else
+    return Argument();
+#endif
 }
 
 Argument Option::createStringListArgument(const std::vector<std::string> &value) const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     return Argument(comp.lock(), opt, make_argument(GPGME_CONF_STRING, value), true);
+#else
+    return Argument();
+#endif
 }
 
 Argument Option::createIntListArgument(const std::vector<int> &value) const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     return Argument(comp.lock(), opt, make_argument(GPGME_CONF_INT32, value), true);
+#else
+    return Argument();
+#endif
 }
 
 Argument Option::createUIntListArgument(const std::vector<unsigned int> &value) const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     return Argument(comp.lock(), opt, make_argument(GPGME_CONF_UINT32, value), true);
+#else
+    return Argument();
+#endif
 }
 
 Argument::Argument(const shared_gpgme_conf_comp_t &comp, gpgme_conf_opt_t opt, gpgme_conf_arg_t arg, bool owns)
     : comp(comp),
       opt(opt),
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
       arg(owns ? arg : mygpgme_conf_arg_copy(arg, opt ? opt->alt_type : GPGME_CONF_NONE))
+#else
+      arg(0)
+#endif
 {
 
 }
@@ -506,14 +825,20 @@ Argument::Argument(const shared_gpgme_conf_comp_t &comp, gpgme_conf_opt_t opt, g
 Argument::Argument(const Argument &other)
     : comp(other.comp),
       opt(other.opt),
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
       arg(mygpgme_conf_arg_copy(other.arg, opt ? opt->alt_type : GPGME_CONF_NONE))
+#else
+      arg(0)
+#endif
 {
 
 }
 
 Argument::~Argument()
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     gpgme_conf_arg_release(arg, opt ? opt->alt_type : GPGME_CONF_NONE);
+#endif
 }
 
 Option Argument::parent() const
@@ -532,27 +857,34 @@ unsigned int Argument::numElements() const
         return 0;
     }
     unsigned int result = 0;
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     for (gpgme_conf_arg_t a = arg ; a ; a = a->next) {
         ++result;
     }
+#endif
     return result;
 }
 
 const char *Argument::stringValue(unsigned int idx) const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull() || opt->alt_type != GPGME_CONF_STRING) {
-        return nullptr;
+        return 0;
     }
     gpgme_conf_arg_t a = arg;
     while (a && idx) {
         a = a->next;
         --idx;
     }
-    return a ? a->value.string : nullptr ;
+    return a ? a->value.string : 0 ;
+#else
+    return 0;
+#endif
 }
 
 int Argument::intValue(unsigned int idx) const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull() || opt->alt_type != GPGME_CONF_INT32) {
         return 0;
     }
@@ -562,10 +894,14 @@ int Argument::intValue(unsigned int idx) const
         --idx;
     }
     return a ? a->value.int32 : 0 ;
+#else
+    return 0;
+#endif
 }
 
 unsigned int Argument::uintValue(unsigned int idx) const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull() || opt->alt_type != GPGME_CONF_UINT32) {
         return 0;
     }
@@ -575,14 +911,21 @@ unsigned int Argument::uintValue(unsigned int idx) const
         --idx;
     }
     return a ? a->value.uint32 : 0 ;
+#else
+    return 0;
+#endif
 }
 
 unsigned int Argument::numberOfTimesSet() const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull() || opt->alt_type != GPGME_CONF_NONE) {
         return 0;
     }
     return arg->value.count;
+#else
+    return 0;
+#endif
 }
 
 std::vector<const char *> Argument::stringValues() const
@@ -599,25 +942,33 @@ std::vector<const char *> Argument::stringValues() const
 
 std::vector<int> Argument::intValues() const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull() || opt->alt_type != GPGME_CONF_INT32) {
         return std::vector<int>();
     }
+#endif
     std::vector<int> result;
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     for (gpgme_conf_arg_t a = arg ; a ; a = a->next) {
         result.push_back(a->value.int32);
     }
+#endif
     return result;
 }
 
 std::vector<unsigned int> Argument::uintValues() const
 {
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     if (isNull() || opt->alt_type != GPGME_CONF_UINT32) {
         return std::vector<unsigned int>();
     }
+#endif
     std::vector<unsigned int> result;
+#ifdef HAVE_GPGME_PROTOCOL_GPGCONF
     for (gpgme_conf_arg_t a = arg ; a ; a = a->next) {
         result.push_back(a->value.uint32);
     }
+#endif
     return result;
 }
 
@@ -750,14 +1101,14 @@ std::ostream &Configuration::operator<<(std::ostream &os, const Argument &a)
                 os << v.size() << ':';
                 // can't use std::copy + ostream_iterator here, since we need the protect() call
                 bool first = true;
-                std::for_each(v.begin(), v.end(), [&first, &os](const char *s) {
+                BOOST_FOREACH(const char *s, v) {
                     if (first) {
                         first = false;
                     } else {
                         os << ',';
                     }
                     os << protect(s);
-                });
+                }
             } else {
                 os << protect(a.stringValue());
             }
